/*
 * Copyright (c) 2002
 * Neil Spring 
 * GPL
 */

#ifndef _NS_COMMON_H
#define _NS_COMMON_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef __cplusplus
# define NS_BEGIN_DECLS extern "C" {
# define NS_END_DECLS }
#else
# define NS_BEGIN_DECLS /* empty */
# define NS_END_DECLS /* empty */
#endif

NS_BEGIN_DECLS

#ifdef __LCLINT__
#define UNUSED(x) /*@unused@*/ ##x
typedef	unsigned int u_int32_t;
#else
#define UNUSED(x) x __attribute__((unused))
#endif

#ifdef __LCLINT__
#define __STDC__
char *rindex(const char *s, char c); /* not really true; it's an int */
/*@dependent@*/char *index(const char *s, char c); /* not really true; it's an int */
char *strdup(const char *s);  /* probably a good annotation here */
/*@printflike@*/
int /*@alt void@*/ asprintf(/*@out@*/char **s, const char *fmt, ...);
/*@printflike@*/
int /*@alt void@*/ snprintf(/*@out@*/char *s, size_t x, const char *fmt, ...);
#endif

typedef unsigned char boolean;

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE (!FALSE)
#endif

#ifdef __LCLINT__
/* lclint doesn't do typeof */
#define min(x,y) ((x > y) ? x : y)
#define max(x,y) ((x > y) ? x : y)
#else
/* from linux/kernel.h */
#define min(x,y) ({ \
        const typeof(x) _xi = (x);       \
        const typeof(y) _yi = (y);       \
        (void) (&_xi == &_yi);            \
        _xi < _yi ? _xi : _yi; })
#define max(x,y) ({ \
        const typeof(x) _xa = (x);       \
        const typeof(y) _ya = (y);       \
        (void) (&_xa == &_ya);            \
        _xa > _ya ? _xa : _ya; })
#endif

#define DEBUG_PRINT(x...) do { } while (0)
// #define DEBUG_PRINT(x) fprintf(stderr, "%s:%d: %s\n", __FILE__, __LINE__, x);
#define DEBUG_PRINTL(x) fprintf(stderr, "%s:%d: %s", __FILE__, __LINE__, x);

#ifdef HAVE_STDINT_H
/* a linux-y thing */
#include <stdint.h>
#else
#include <sys/types.h>
#endif
typedef uint64_t macaddress;

#ifdef VERBOSE
#define VERBAL printf
#else
#define VERBAL(X...) do { if(want_debug) fprintf(stderr, X); } while(0);

#if defined(__linux__)  && !defined(__cplusplus)
/*@unused@*/
static __inline void nullie(/*@unused@*/ const char *fmt __attribute__ ((unused)), ...) {
  return;
}
#else
#define nullie(x...) do {} while(0)
#endif

#endif

#include<stdlib.h>
#include<stdio.h>
/*@unused@*/
static __inline /*@out@*/ void *malloc_ordie(size_t len) {
  void *ret = malloc(len);
  if(ret == NULL) { 
    fprintf(stderr, "unable to allocate %d bytes\n", (int)len);
    abort();
  }
  return(ret);
}

#define NEWPTR(type, name) type *name = (type *)malloc_ordie(sizeof(type))
#define NEWPTRX(type, name) type *name = (type *)malloc_ordie(sizeof(type))
#define NEWPTRZ(type, name) type *name = (type *)memset(malloc_ordie(sizeof(type)), 0, sizeof(type))

NS_END_DECLS

#endif
/*
vi:ts=2
*/
